SystemOrganization addCategory: #PasswordHashingFFI!

ExternalStructure subclass: #BCryptLinuxFFI
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PasswordHashingFFI'!

!BCryptLinuxFFI class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 5/1/2011 14:15'!
bcrypt: password
	"Returns the bcrypt encrypted password.  "
	^self bcrypt: password with: self randomBCryptSaltData.   
	! !

!BCryptLinuxFFI class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 5/1/2011 14:14'!
bcrypt: password with: salt
	"Returns the bcrypt encrypted password.  "
	^self bcrypt: password with: salt  for: self defaultRounds.   
	! !

!BCryptLinuxFFI class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 5/1/2011 15:41'!
bcrypt: password with: salt for: iterations


	"Returns the bcrypt encrypted password. use salt=  nil for random salt.    "
	| bcryptSalt  |
	
	bcryptSalt :='$2a$', (self generateBCryptIterations: iterations) , '$', (self generateBCryptSalt: salt).

	^self ffiCrypt: password with: bcryptSalt.
	! !

!BCryptLinuxFFI class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 5/2/2011 15:37'!
bcrypt: password withSalt: salt
	"Returns the bcrypt encrypted password.  "
	^self ffiCrypt: password  with:   salt
	! !

!BCryptLinuxFFI class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 5/3/2011 11:21'!
checkPassword: password against: hash
|salt| 
salt := hash copyFrom: 1 to:30.

^hash = (self ffiCrypt: password with: salt).! !

!BCryptLinuxFFI class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 4/27/2011 11:33'!
defaultRounds
	^10! !

!BCryptLinuxFFI class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 5/2/2011 16:58'!
ffiCrypt: password with: key  
" on linux you need to install and use the 32bit version of libxcrypt and type its path in the module: field below."
<cdecl: char* 'crypt' (char* char*) module: '/emul/ia32-linux/lib/libxcrypt.so.1'>
^self externalCallFailed! !

!BCryptLinuxFFI class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 5/1/2011 15:25'!
generateBCryptIterations: anInteger
	| max min num |
	max := 31.
	min := 4.
	num := anInteger <= min
		ifTrue: [ min ]
		ifFalse: [
			anInteger >= max
				ifTrue: [ max ]
				ifFalse: [ anInteger ] ].
			
	^num<10 ifTrue:['0' , num greaseString ] ifFalse:[num greaseString]! !

!BCryptLinuxFFI class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 5/1/2011 14:22'!
generateBCryptSalt: aSalt
	| newSalt saltSize |
	saltSize := 22.
	aSalt == nil
		ifTrue: [ newSalt := self randomBCryptSalt: saltSize ]
		ifFalse: [ 
			aSalt size < saltSize
				ifTrue: [ 
					newSalt := String new: saltSize.
					(1 to: saltSize) do: [ :each | newSalt at: each put: (aSalt atWrap: each) ] ]
				ifFalse: [ newSalt := aSalt copyFrom: 1 to: saltSize ] ].
	^ newSalt! !

!BCryptLinuxFFI class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 5/1/2011 14:12'!
generateRandomSalt

	^'$2a$' , self defaultRounds greaseString , '$' , self randomBCryptSaltData.! !

!BCryptLinuxFFI class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 5/1/2011 15:24'!
iterationString: num
			
	^num<10 ifTrue:['0' , num greaseString ] ifFalse:[num greaseString]! !

!BCryptLinuxFFI class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 5/1/2011 14:13'!
randomBCryptSaltData
	 
	| numbers bigAlphas littleAlphas combined targetStream char random |
	numbers := (46 to: 57) collect: [:each | each asCharacter].
	bigAlphas := (65 to: 90) collect: [:each | each asCharacter].
	littleAlphas := (97 to: 122) collect: [:each | each asCharacter].
	combined:= numbers, bigAlphas, littleAlphas.
	targetStream := WriteStream on: (String new: self saltDataLength ).
	random := Random new.
	[targetStream contents size <= self saltDataLength] whileTrue: 
			[char :=combined 
						at: (random next * (combined size - 1)) rounded + 1.
			char isAlphaNumeric ifTrue: [targetStream nextPut: char]].
	^targetStream contents  ! !

!BCryptLinuxFFI class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 4/27/2011 11:33'!
saltDataLength
	^22! !

!BCryptLinuxFFI class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 4/27/2011 11:33'!
saltOutputLength
	^30! !

ExternalStructure subclass: #CryptLinuxFFI
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PasswordHashingFFI'!

!CryptLinuxFFI class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 5/2/2011 08:59'!
checkPassword: password against: hashedPassword
	| salt newHash |
	salt := (hashedPassword at: 1) = $$
		ifTrue: [ hashedPassword copyFrom: 1 to: 12 ]
		ifFalse: [ hashedPassword copyFrom: 1 to: 2 ].
		
	newHash := self ffiCrypt: password  with:   salt.
	^newHash = hashedPassword ! !

!CryptLinuxFFI class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 5/2/2011 08:24'!
des: password with: salt
	"Returns the DES encrypted password.  "

	^self ffiCrypt: password with: (self prepareSalt:   salt).
	! !

!CryptLinuxFFI class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 5/2/2011 08:03'!
ffiCrypt: aKey with: aSalt
<cdecl: char* 'crypt' (char* char*) module: 'crypt'>
^self externalCallFailed! !

!CryptLinuxFFI class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 5/2/2011 08:24'!
md5: password with: salt
	"Returns the md5 hashed password.  "
	|cryptSalt|
	cryptSalt :='$1$',  (self prepareSalt:   salt) , '$'.
	^self ffiCrypt: password with: cryptSalt.
	! !

!CryptLinuxFFI class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 5/2/2011 08:50'!
prepareSalt: aSalt
"the crypt(3) function only uses the first 8 characters of the salt.  "
| newSalt saltSize |
	saltSize := 8.
	aSalt == nil
		ifTrue: [ newSalt := self randomSalt: saltSize ]
		ifFalse: [ 
			aSalt size < saltSize
				ifTrue: [ 
					newSalt := String new: saltSize.
					(1 to: saltSize) do: [ :each | newSalt at: each put: (aSalt atWrap: each) ] ]
				ifFalse: [ newSalt := aSalt copyFrom: 1 to: saltSize ] ].
	^ newSalt! !

!CryptLinuxFFI class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 5/2/2011 08:23'!
randomSalt: aSize
	| numbers bigAlphas littleAlphas combined targetStream char random |
	numbers := (46 to: 57) collect: [:each | each asCharacter].
	bigAlphas := (65 to: 90) collect: [:each | each asCharacter].
	littleAlphas := (97 to: 122) collect: [:each | each asCharacter].
	combined:= numbers, bigAlphas, littleAlphas.
	targetStream := WriteStream on: (String new: aSize ).
	random := Random new.
	[targetStream contents size <= aSize] whileTrue: 
			[char :=combined 
						at: (random next * (combined size - 1)) rounded + 1.
			char isAlphaNumeric ifTrue: [targetStream nextPut: char]].
	^targetStream contents  ! !

!CryptLinuxFFI class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 5/2/2011 08:24'!
sha256: password with: salt
	"Returns the sha256 hashed password.  "
	|cryptSalt|
	cryptSalt :='$5$',  (self prepareSalt:   salt) , '$'.
	^self ffiCrypt: password with: cryptSalt.
	! !

!CryptLinuxFFI class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 5/2/2011 08:24'!
sha512: password with: salt
	"Returns thesha512 hashed password.  "
	|cryptSalt|
	cryptSalt :='$6$',  (self prepareSalt:   salt) , '$'.
	^self ffiCrypt: password with: cryptSalt.
	! !

TestCase subclass: #PasswordHashingFFITests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'PasswordHashingFFI'!

!PasswordHashingFFITests methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 5/2/2011 08:23'!
testBcryptFfi
	self assert:(( BCryptLinuxFFI  bcrypt: 'U*U' with:'C'  for: 5) = '$2a$05$CCCCCCCCCCCCCCCCCCCCC.E5YPO9kmyuRGyh0XouQYb4YMJKvyOeW')! !

!PasswordHashingFFITests methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 5/2/2011 08:01'!
testFfiBcrypt
	self assert:(( BCryptLinuxFFI  bcrypt: 'U*U' with:'C'  for: 5) = '$2a$05$CCCCCCCCCCCCCCCCCCCCC.E5YPO9kmyuRGyh0XouQYb4YMJKvyOeW')! !
